@@ -183,8 +183,6 @@ def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
         elif not newenv:
             if context.json:
                 cli_common.stdout_json_success(message='All requested packages already installed.')
-            else:
-                print('\n# All requested packages already installed.\n')
             return
 
     if context.dry_run:
@@ -271,8 +269,6 @@ def remove(args, parser):
             specs = [s for s in specs_from_args(args.package_names)]
         if not context.quiet:
             print("Removing specs: {}".format([s.conda_build_form() for s in specs]))
-        channel_urls = ()
-        subdirs = ()
 
         installed_json_f = get_installed_jsonfile(prefix)
 
@@ -302,7 +298,8 @@ def remove(args, parser):
             exit_code = 1
             return exit_code
 
-        transaction = api.Transaction(solver)
+        package_cache = api.MultiPackageCache(context.pkgs_dirs)
+        transaction = api.Transaction(solver, package_cache)
         to_link, to_unlink = transaction.to_conda()
         transaction.log_json()
 
@@ -362,7 +359,6 @@ def install(args, parser, command='install'):
             else:
                 raise EnvironmentLocationNotFound(prefix)
 
-    prepend = not args.override_channels
     prefix = context.target_prefix
 
     index_args = {
@@ -510,7 +506,6 @@ def install(args, parser, command='install'):
         repo.set_installed()
         repos.append(repo)
 
-
     for channel, cache_file, priority, subpriority in channel_json:
         repo = api.Repo(pool, str(channel), cache_file, channel.url(with_credentials=True))
         repo.set_priority(priority, subpriority)
@@ -524,7 +519,8 @@ def install(args, parser, command='install'):
         exit_code = 1
         return exit_code
 
-    transaction = api.Transaction(solver)
+    package_cache = api.MultiPackageCache(context.pkgs_dirs)
+    transaction = api.Transaction(solver, package_cache)
     to_link, to_unlink = transaction.to_conda()
     transaction.log_json()
 
@@ -536,8 +532,13 @@ def install(args, parser, command='install'):
     if python_added:
         specs = [s for s in specs if s.name != 'python']
 
-    conda_transaction = to_txn(specs, (), prefix, to_link, to_unlink, index)
-    handle_txn(conda_transaction, prefix, args, newenv)
+    if use_mamba_experimental and not os.name == 'nt':
+        if command == 'create' and not isdir(context.target_prefix):
+            mkdir_p(prefix)
+        transaction.execute(prefix_data, PackageCacheData.first_writable().pkgs_dir)
+    else:
+        conda_transaction = to_txn(specs, (), prefix, to_link, to_unlink, index)
+        handle_txn(conda_transaction, prefix, args, newenv)
 
     try:
         installed_json_f.close()
@@ -575,7 +576,6 @@ def update(args, parser):
     install(args, parser, 'update')
 
 def repoquery(args, parser):
-    prepend = not args.override_channels
     prefix = context.target_prefix
 
     init_api_context()
@@ -598,7 +598,11 @@ def repoquery(args, parser):
     repo.set_installed()
     repos.append(repo)
 
-    if not args.installed:
+    only_installed = True
+    if args.subcmd == "search" and args.installed == False:
+        only_installed = False
+
+    if not only_installed:
         index = get_index(channel_urls=index_args['channel_urls'],
                       prepend=index_args['prepend'], platform=None,
                       use_local=index_args['use_local'], use_cache=index_args['use_cache'],
@@ -614,15 +618,15 @@ def repoquery(args, parser):
             repos.append(repo)
 
 
-    print("\nExecuting the query %s\n" % args.query)
+    print("\nExecuting the query %s\n" % args.package_query)
 
     query = api.Query(pool)
-    if args.whatrequires:
-        print(query.whatrequires(args.query))
-    elif args.tree:
-        print(query.dependencytree(args.query))
-    else:
-        print(query.find(args.query))
+    if args.subcmd == "whoneeds":
+        print(query.whatrequires(args.package_query, args.tree))
+    if args.subcmd == "depends":
+        print(query.dependencytree(args.package_query))
+    if args.subcmd == "search":
+        print(query.find(args.package_query))
 
 
 def do_call(args, parser):
@@ -656,42 +660,49 @@ def configure_parser_repoquery(sub_parsers):
     example = ("""
 Examples:
 
-    conda repoquery xtensor>=0.18
+    mamba repoquery search xtensor>=0.18
+    mamba repoquery depends xtensor
+    mamba repoquery whoneeds xtl
 
     """)
 
+    import argparse
     from argparse import SUPPRESS
 
     p = sub_parsers.add_parser(
         'repoquery',
         description=descr,
         help=help,
-        epilog=example,
+        epilog=example
     )
-
-    p.add_argument(
-        'query',
-        action="store",
-        nargs='?',
-        help="Package query.",
-    )
-
-    p.add_argument(
-        "--whatrequires",
+    subsub_parser = p.add_subparsers(dest='subcmd')
+    package_cmds = argparse.ArgumentParser(add_help=False)
+    package_cmds.add_argument('package_query', help='the target package')
+    package_cmds.add_argument(
+        "-i", "--installed",
         action="store_true",
-        help=SUPPRESS,
+        help=SUPPRESS
     )
 
-    p.add_argument(
-        "--installed",
-        action="store_true",
-        help=SUPPRESS,
+    view_cmds = argparse.ArgumentParser(add_help=False)
+    view_cmds.add_argument(
+        "-t", "--tree",
+        action="store_true"
     )
 
-    p.add_argument(
-        "--tree",
-        action="store_true",
-        help=SUPPRESS,
+    subsub_parser.add_parser('whoneeds',
+        help='shows packages that depends on this package',
+        parents=[package_cmds, view_cmds]
+    )
+
+    subsub_parser.add_parser('depends',
+        help='shows packages that depends on this package',
+        parents=[package_cmds, view_cmds]
+    )
+
+    subsub_parser.add_parser('search',
+        help='shows packages that depends on this package',
+        parents=[package_cmds]
     )
 
     from conda.cli import conda_argparse
@@ -702,7 +713,6 @@ Examples:
     p.set_defaults(func='.main_repoquery.execute')
     return p
 
-
 def _wrapped_main(*args, **kwargs):
     if len(args) == 1:
         args = args + ('-h',)
